<?php
/**
 * 2007-2020 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 *
 *
 * @author    yourName
 * @copyright 2007-2020
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *
 * @generated by PrestashopConsole  / copyright 2016-2020 Hennes Hervé http://www.h-hennes.fr/blog/
 */

if (!defined('_PS_VERSION_')) {
    exit;
}


/**
 * Class Hh_clicandcollectdelivery
 * Définition d'une zone personnnalisée en fonction d'un code postal
 */
class Hh_clicandcollectdelivery extends Module
{

    /** @var string Clé de configuration */
    const CONFIGURATION_KEYS = [
        'HH_CLICANDCOLLECT_ENABLE',
        'HH_CLICANDCOLLECT_ALLOWED_POSTCODES',
        'HH_CLICANDCOLLECT_ZONE_ID'
    ];

    public function __construct()
    {
        $this->name = 'hh_clicandcollectdelivery';
        $this->tab = 'others';
        $this->version = '0.1.0';
        $this->author = 'hhennes';
        $this->bootstrap = true;
        parent::__construct();

        $this->displayName = $this->l('Hh Clic and collect delivery');
        $this->description = $this->l('Allow clic and collect to certain postcode');
    }

    /**
     * Installation du module
     * @return bool
     */
    public function install()
    {
        if (
            !parent::install()
            || !$this->registerHook('actionGetIDZoneByAddressID')
            || !$this->_installZone()
        ) {
            return false;
        }

        return true;
    }

    /**
     * Création de la zone clic and collect
     * @return bool
     */
    protected function _installZone()
    {
        try {
            $zone = new Zone();
            $zone->name = $this->l('Clic and collect zone');
            $zone->save();
            Configuration::updateValue('HH_CLICANDCOLLECT_ZONE_ID', $zone->id);
        } catch (PrestaShopException $e) {
            return false;
        }
        return true;
    }

    /**
     * Désintallation module
     * @return bool
     */
    public function uninstall()
    {
        return parent::uninstall() && $this->_removeConfiguration();
    }

    /**
     * Soumission formulaire de configuration
     * @return string|void
     */
    protected function postProcess()
    {
        if (Tools::isSubmit('SubmitConfigForm')) {
            $success = true;
            foreach (self::CONFIGURATION_KEYS as $key) {
                if (!Configuration::updateValue($key, Tools::getValue($key))) {
                    $success = false;
                }
            }
            if (true === $success) {
                return $this->displayConfirmation($this->l('Settings Updated'));
            } else {
                return $this->displayError($this->l('Unable to update settings'));
            }
        }
    }

    /**
     * @return string
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    public function getContent()
    {
        $html = $this->postProcess();
        $html .= $this->renderForm();

        return $html;
    }

    /**
     * Get configuration form
     * @return string
     * @throws PrestaShopDatabaseException
     * @throws PrestaShopException
     */
    protected function renderForm()
    {
        $fields_form = [
            'form' => [
                'legend' => [
                    'title' => $this->l('Configure click and collect'),
                    'icon' => 'icon-cogs'
                ],
                'input' => [
                    [
                        'type' => 'switch',
                        'label' => $this->l('Enable Click and collect'),
                        'name' => 'HH_CLICANDCOLLECT_ENABLE',
                        'required' => true,
                        'values' => [
                            ['id' => 'on', 'value' => 1, 'label' => $this->l('Yes')],
                            ['id' => 'off', 'value' => 0, 'label' => $this->l('No')],
                        ],
                    ],
                    [
                        'type' => 'text',
                        'label' => $this->l('Enable for postcodes'),
                        'name' => 'HH_CLICANDCOLLECT_ALLOWED_POSTCODES',
                        'hint' => $this->l('Define postcode with a regexp like ^75 or ^67000$ or ^6(7|8)'),
                        'required' => true,
                    ],
                    [
                        'type' => 'select',
                        'label' => $this->l('Related zone'),
                        'name' => 'HH_CLICANDCOLLECT_ZONE_ID',
                        'hint' => $this->l('Define the prestashop zone which delivery condition will apply to customer'),
                        'required' => true,
                        'options' => [
                            'query' => Zone::getZones(true),
                            'id' => 'id_zone',
                            'name' => 'name'
                        ],
                    ],

                ],
                'submit' => [
                    'title' => $this->l('Save'),
                    'class' => 'button btn btn-default pull-right',
                ]
            ],
        ];

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $lang = new Language((int)Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $helper->identifier = $this->name;
        $helper->submit_action = 'SubmitConfigForm';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => [
                'HH_CLICANDCOLLECT_ENABLE' => Configuration::get('HH_CLICANDCOLLECT_ENABLE',
                    Tools::getValue('HH_CLICANDCOLLECT_ENABLE')),
                'HH_CLICANDCOLLECT_ALLOWED_POSTCODES' => Configuration::get('HH_CLICANDCOLLECT_ALLOWED_POSTCODES',
                    Tools::getValue('HH_CLICANDCOLLECT_ALLOWED_POSTCODES')),
                'HH_CLICANDCOLLECT_ZONE_ID' => Configuration::get('HH_CLICANDCOLLECT_ZONE_ID',
                    Tools::getValue('HH_CLICANDCOLLECT_ZONE_ID')),
            ],
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id
        );

        return $helper->generateForm(array($fields_form));
    }

    /**
     * Suppression des clés de configuration
     * @return bool
     */
    protected function _removeConfiguration()
    {
        foreach (self::CONFIGURATION_KEYS as $key) {
            if (Configuration::get($key)) {
                if (!Configuration::deleteByName($key)) {
                    return false;
                }
            }
        }
        return true;
    }

    /**
     * Récupération de l'identifiant de la zone
     * @param array $params
     * @return mixed
     */
    public function hookActionGetIDZoneByAddressID($params)
    {
        if (Configuration::get('HH_CLICANDCOLLECT_ENABLE')) {
            $address = new Address($params['id_address']);
            if (preg_match('#' . Configuration::get('HH_CLICANDCOLLECT_ALLOWED_POSTCODES') . '#', $address->postcode)) {
                return Configuration::get('HH_CLICANDCOLLECT_ZONE_ID');
            }
        }
    }
}
